@@ -1,13 +1,11 @@
 /*
    +----------------------------------------------------------------------+
-   | PHP Version 7                                                        |
-   +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2018 The PHP Group                                |
+   | Copyright (c) The PHP Group                                          |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
    | available through the world-wide-web at the following url:           |
-   | http://www.php.net/license/3_01.txt                                  |
+   | https://www.php.net/license/3_01.txt                                 |
    | If you did not receive a copy of the PHP license and are unable to   |
    | obtain it through the world-wide-web, please send a note to          |
    | license@php.net so we can mail you a copy immediately.               |
@@ -22,7 +20,7 @@
 #include "php_version.h"
 #define PHP_CTYPE_VERSION PHP_VERSION
 
-#if HAVE_CTYPE
+#ifdef HAVE_CTYPE
 
 extern zend_module_entry ctype_module_entry;
 #define phpext_ctype_ptr &ctype_module_entry
@@ -34,11 +32,3 @@ extern zend_module_entry ctype_module_entry;
 #endif
 
 #endif	/* PHP_CTYPE_H */
-
-
-/*
- * Local variables:
- * tab-width: 4
- * c-basic-offset: 4
- * End:
- */
