@@ -1,13 +1,11 @@
 /*
    +----------------------------------------------------------------------+
-   | PHP Version 5                                                        |
-   +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2015 The PHP Group                                |
+   | Copyright (c) The PHP Group                                          |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
    | available through the world-wide-web at the following url:           |
-   | http://www.php.net/license/3_01.txt                                  |
+   | https://www.php.net/license/3_01.txt                                 |
    | If you did not receive a copy of the PHP license and are unable to   |
    | obtain it through the world-wide-web, please send a note to          |
    | license@php.net so we can mail you a copy immediately.               |
@@ -17,9 +15,9 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id$ */
-
-#define _GNU_SOURCE
+#ifndef _GNU_SOURCE
+# define _GNU_SOURCE
+#endif
 
 #include "php.h"
 #include "php_zlib.h"
@@ -32,50 +30,85 @@ struct php_gz_stream_data_t	{
 	php_stream *stream;
 };
 
-static size_t php_gziop_read(php_stream *stream, char *buf, size_t count TSRMLS_DC)
+static ssize_t php_gziop_read(php_stream *stream, char *buf, size_t count)
 {
 	struct php_gz_stream_data_t *self = (struct php_gz_stream_data_t *) stream->abstract;
-	int read;
-	
-	read = gzread(self->gz_file, buf, count);
-	
-	if (gzeof(self->gz_file)) {
-		stream->eof = 1;
-	}
-		
-	return (read < 0) ? 0 : read;
+	ssize_t total_read = 0;
+
+	/* Despite the count argument of gzread() being "unsigned int",
+	 * the return value is "int". Error returns are values < 0, otherwise the count is returned.
+	 * To properly distinguish error values from success value, we therefore need to cap at INT_MAX.
+	 */
+	do {
+		unsigned int chunk_size = MIN(count, INT_MAX);
+		int read = gzread(self->gz_file, buf, chunk_size);
+		count -= chunk_size;
+
+		if (gzeof(self->gz_file)) {
+			stream->eof = 1;
+		}
+
+		if (UNEXPECTED(read < 0)) {
+			return read;
+		}
+
+		total_read += read;
+		buf += read;
+	} while (count > 0 && !stream->eof);
+
+	return total_read;
 }
 
-static size_t php_gziop_write(php_stream *stream, const char *buf, size_t count TSRMLS_DC)
+static ssize_t php_gziop_write(php_stream *stream, const char *buf, size_t count)
 {
 	struct php_gz_stream_data_t *self = (struct php_gz_stream_data_t *) stream->abstract;
-	int wrote;
+	ssize_t total_written = 0;
 
-	wrote = gzwrite(self->gz_file, (char *) buf, count);
+	/* Despite the count argument of gzread() being "unsigned int",
+	 * the return value is "int". Error returns are values < 0, otherwise the count is returned.
+	 * To properly distinguish error values from success value, we therefore need to cap at INT_MAX.
+	 */
+	do {
+		unsigned int chunk_size = MIN(count, INT_MAX);
+		int written = gzwrite(self->gz_file, buf, chunk_size);
+		count -= chunk_size;
 
-	return (wrote < 0) ? 0 : wrote;
+		if (UNEXPECTED(written < 0)) {
+			return written;
+		}
+
+		total_written += written;
+		buf += written;
+	} while (count > 0);
+
+    return total_written;
 }
 
-static int php_gziop_seek(php_stream *stream, off_t offset, int whence, off_t *newoffs TSRMLS_DC)
+static int php_gziop_seek(php_stream *stream, zend_off_t offset, int whence, zend_off_t *newoffs)
 {
 	struct php_gz_stream_data_t *self = (struct php_gz_stream_data_t *) stream->abstract;
 
 	assert(self != NULL);
 
 	if (whence == SEEK_END) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "SEEK_END is not supported");
+		php_error_docref(NULL, E_WARNING, "SEEK_END is not supported");
 		return -1;
 	}
-	*newoffs = gzseek(self->gz_file, offset, whence);
 
-	return (*newoffs < 0) ? -1 : 0;
+	z_off_t new_offset = gzseek(self->gz_file, offset, whence);
+	if (new_offset < 0) {
+		return -1;
+	}
+
+	*newoffs = new_offset;
+	return 0;
 }
 
-static int php_gziop_close(php_stream *stream, int close_handle TSRMLS_DC)
+static int php_gziop_close(php_stream *stream, int close_handle)
 {
 	struct php_gz_stream_data_t *self = (struct php_gz_stream_data_t *) stream->abstract;
 	int ret = EOF;
-	
+
 	if (close_handle) {
 		if (self->gz_file) {
 			ret = gzclose(self->gz_file);
@@ -91,25 +124,25 @@ static int php_gziop_close(php_stream *stream, int close_handle TSRMLS_DC)
 	return ret;
 }
 
-static int php_gziop_flush(php_stream *stream TSRMLS_DC)
+static int php_gziop_flush(php_stream *stream)
 {
 	struct php_gz_stream_data_t *self = (struct php_gz_stream_data_t *) stream->abstract;
 
 	return gzflush(self->gz_file, Z_SYNC_FLUSH);
 }
 
-php_stream_ops php_stream_gzio_ops = {
+const php_stream_ops php_stream_gzio_ops = {
 	php_gziop_write, php_gziop_read,
 	php_gziop_close, php_gziop_flush,
 	"ZLIB",
-	php_gziop_seek, 
+	php_gziop_seek,
 	NULL, /* cast */
 	NULL, /* stat */
 	NULL  /* set_option */
 };
 
-php_stream *php_stream_gzopen(php_stream_wrapper *wrapper, const char *path, const char *mode, int options, 
-							  char **opened_path, php_stream_context *context STREAMS_DC TSRMLS_DC)
+php_stream *php_stream_gzopen(php_stream_wrapper *wrapper, const char *path, const char *mode, int options,
+							  zend_string **opened_path, php_stream_context *context STREAMS_DC)
 {
 	struct php_gz_stream_data_t *self;
 	php_stream *stream = NULL, *innerstream = NULL;
@@ -117,19 +150,19 @@ php_stream *php_stream_gzopen(php_stream_wrapper *wrapper, const char *path, con
 	/* sanity check the stream: it can be either read-only or write-only */
 	if (strchr(mode, '+')) {
 		if (options & REPORT_ERRORS) {
-			php_error_docref(NULL TSRMLS_CC, E_WARNING, "cannot open a zlib stream for reading and writing at the same time!");
+			php_error_docref(NULL, E_WARNING, "Cannot open a zlib stream for reading and writing at the same time!");
 		}
 		return NULL;
 	}
-	
+
 	if (strncasecmp("compress.zlib://", path, 16) == 0) {
 		path += 16;
 	} else if (strncasecmp("zlib:", path, 5) == 0) {
 		path += 5;
 	}
-	
+
 	innerstream = php_stream_open_wrapper_ex(path, mode, STREAM_MUST_SEEK | options | STREAM_WILL_CAST, opened_path, context);
-	
+
 	if (innerstream) {
 		php_socket_t fd;
 
@@ -139,6 +172,11 @@ php_stream *php_stream_gzopen(php_stream_wrapper *wrapper, const char *path, con
 			self->gz_file = gzdopen(dup(fd), mode);
 
 			if (self->gz_file) {
+				zval *zlevel = context ? php_stream_context_get_option(context, "zlib", "level") : NULL;
+				if (zlevel && (Z_OK != gzsetparams(self->gz_file, zval_get_long(zlevel), Z_DEFAULT_STRATEGY))) {
+					php_error(E_WARNING, "failed setting compression level");
+				}
+
 				stream = php_stream_alloc_rel(&php_stream_gzio_ops, self, 0, mode);
 				if (stream) {
 					stream->flags |= PHP_STREAM_FLAG_NO_BUFFER;
@@ -150,7 +188,7 @@ php_stream *php_stream_gzopen(php_stream_wrapper *wrapper, const char *path, con
 
 			efree(self);
 			if (options & REPORT_ERRORS) {
-				php_error_docref(NULL TSRMLS_CC, E_WARNING, "gzopen failed");
+				php_error_docref(NULL, E_WARNING, "gzopen failed");
 			}
 		}
 
@@ -160,7 +198,7 @@ php_stream *php_stream_gzopen(php_stream_wrapper *wrapper, const char *path, con
 	return NULL;
 }
 
-static php_stream_wrapper_ops gzip_stream_wops = {
+static const php_stream_wrapper_ops gzip_stream_wops = {
 	php_stream_gzopen,
 	NULL, /* close */
 	NULL, /* stat */
@@ -170,20 +208,12 @@ static php_stream_wrapper_ops gzip_stream_wops = {
 	NULL, /* unlink */
 	NULL, /* rename */
 	NULL, /* mkdir */
-	NULL  /* rmdir */
+	NULL, /* rmdir */
+	NULL
 };
 
-php_stream_wrapper php_stream_gzip_wrapper =	{
+const php_stream_wrapper php_stream_gzip_wrapper =	{
 	&gzip_stream_wops,
 	NULL,
 	0, /* is_url */
 };
-
-/*
- * Local variables:
- * tab-width: 4
- * c-basic-offset: 4
- * End:
- * vim600: noet sw=4 ts=4 fdm=marker
- * vim<600: noet sw=4 ts=4
- */
